;;; custom-load.el --- automatically extracted custom dependencies

;;; Code:

(custom-add-loads 'extensions '("eldoc"))
(custom-add-loads 'eldoc '("eldoc"))
(custom-add-loads 'ielm '("ielm"))
(custom-add-loads 'lisp '("elp" "find-func" "ielm"))
(custom-add-loads 'elp '("elp"))
(custom-add-loads 'find-function '("find-func"))
(custom-add-loads 'matching '("hide-copyleft"))

;;; custom-load.el ends here
